@@ -20,6 +20,7 @@ from alembic.ddl.base import (
     ColumnType,
     alter_column,
     alter_table,
+    format_server_default,
     format_type,
 )
 from google.api_core.client_options import ClientOptions
@@ -71,8 +72,6 @@ def reset_connection(dbapi_conn, connection_record, reset_state=None):
 
         dbapi_conn.staleness = None
         dbapi_conn.read_only = False
-    else:
-        dbapi_conn.rollback()
 
 
 # register a method to get a single value of a JSON object
@@ -427,7 +426,14 @@ class SpannerSQLCompiler(SQLCompiler):
             self._label_select_column(
                 None, c, True, False, {"spanner_is_returning": True}
             )
-            for c in expression._select_iterables(returning_cols)
+            for c in expression._select_iterables(
+                filter(
+                    lambda col: not col.dialect_options.get("spanner", {}).get(
+                        "exclude_from_returning", False
+                    ),
+                    returning_cols,
+                )
+            )
         ]
 
         return "THEN RETURN " + ", ".join(columns)
@@ -574,12 +580,24 @@ class SpannerDDLCompiler(DDLCompiler):
         elif has_identity:
             colspec += " " + self.process(column.identity)
         elif default is not None:
-            colspec += " DEFAULT (" + default + ")"
+            colspec += f" DEFAULT {default}"
         elif hasattr(column, "computed") and column.computed is not None:
             colspec += " " + self.process(column.computed)
 
+        if column.dialect_options.get("spanner", {}).get(
+            "allow_commit_timestamp", False
+        ):
+            colspec += " OPTIONS (allow_commit_timestamp=true)"
+
         return colspec
 
+    def get_column_default_string(self, column):
+        default = super().get_column_default_string(column)
+        if default is not None:
+            return f"({default})"
+
+        return default
+
     def visit_computed_column(self, generated, **kw):
         """Computed column operator."""
         text = "AS (%s) STORED" % self.sql_compiler.process(
@@ -649,6 +667,12 @@ class SpannerDDLCompiler(DDLCompiler):
             "Create UNIQUE indexes instead."
         )
 
+    def visit_foreign_key_constraint(self, constraint, **kw):
+        text = super().visit_foreign_key_constraint(constraint, **kw)
+        if constraint.dialect_options.get("spanner", {}).get("not_enforced", False):
+            text += " NOT ENFORCED"
+        return text
+
     def post_create_table(self, table):
         """Build statements to be executed after CREATE TABLE.
 
@@ -807,16 +831,22 @@ class SpannerDialect(DefaultDialect):
     supports_sequences = True
     sequences_optional = False
     supports_identity_columns = True
-    supports_native_enum = True
     supports_native_boolean = True
     supports_native_decimal = True
     supports_statement_cache = True
+    # Spanner uses protos for enums. Creating a column like
+    # Column("an_enum", Enum("A", "B", "C")) will result in a String
+    # column. Setting supports_native_enum to False allows SQLAlchemy
+    # to generate check constraints to enforce the enum values if the
+    # create_constraint=True flag is passed to the Enum constructor.
+    supports_native_enum = False
 
     postfetch_lastrowid = False
     insert_returning = True
     update_returning = True
     delete_returning = True
     supports_multivalues_insert = True
+    use_insertmanyvalues = True
 
     ddl_compiler = SpannerDDLCompiler
     preparer = SpannerIdentifierPreparer
@@ -1092,7 +1122,8 @@ class SpannerDialect(DefaultDialect):
 
         sql = """
             SELECT col.table_schema, col.table_name, col.column_name,
-                col.spanner_type, col.is_nullable, col.generation_expression
+                col.spanner_type, col.is_nullable, col.generation_expression,
+                col.column_default
             FROM information_schema.columns as col
             JOIN information_schema.tables AS t
                 USING (TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME)
@@ -1120,7 +1151,7 @@ class SpannerDialect(DefaultDialect):
                     "name": col[2],
                     "type": self._designate_type(col[3]),
                     "nullable": col[4] == "YES",
-                    "default": None,
+                    "default": col[6] if col[6] is not None else None,
                 }
 
                 if col[5] is not None:
@@ -1481,11 +1512,11 @@ class SpannerDialect(DefaultDialect):
             )
             FROM information_schema.table_constraints AS tc
             JOIN information_schema.constraint_column_usage AS ccu
-                ON ccu.table_catalog = tc.table_catalog
+                ON ccu.constraint_catalog = tc.table_catalog
                 and ccu.constraint_schema = tc.table_schema
                 and ccu.constraint_name = tc.constraint_name
             JOIN information_schema.constraint_table_usage AS ctu
-                ON ctu.table_catalog = tc.table_catalog
+                ON ctu.constraint_catalog = tc.table_catalog
                 and ctu.constraint_schema = tc.table_schema
                 and ctu.constraint_name = tc.constraint_name
             JOIN information_schema.key_column_usage AS kcu
@@ -1838,11 +1869,14 @@ LIMIT 1
 def visit_column_nullable(
     element: "ColumnNullable", compiler: "SpannerDDLCompiler", **kw
 ) -> str:
-    return "%s %s %s %s" % (
-        alter_table(compiler, element.table_name, element.schema),
-        alter_column(compiler, element.column_name),
-        format_type(compiler, element.existing_type),
-        "" if element.nullable else "NOT NULL",
+    return _format_alter_column(
+        compiler,
+        element.table_name,
+        element.schema,
+        element.column_name,
+        element.existing_type,
+        element.nullable,
+        element.existing_server_default,
     )
 
 
@@ -1851,9 +1885,34 @@ def visit_column_nullable(
 def visit_column_type(
     element: "ColumnType", compiler: "SpannerDDLCompiler", **kw
 ) -> str:
-    return "%s %s %s %s" % (
-        alter_table(compiler, element.table_name, element.schema),
-        alter_column(compiler, element.column_name),
-        "%s" % format_type(compiler, element.type_),
-        "" if element.existing_nullable else "NOT NULL",
+    return _format_alter_column(
+        compiler,
+        element.table_name,
+        element.schema,
+        element.column_name,
+        element.type_,
+        element.existing_nullable,
+        element.existing_server_default,
+    )
+
+
+def _format_alter_column(
+    compiler, table_name, schema, column_name, type_, nullable, server_default
+):
+    # Older versions of SQLAlchemy pass in a boolean to indicate whether there
+    # is an existing DEFAULT constraint, instead of the actual DEFAULT constraint
+    # expression. In those cases, we do not want to explicitly include the DEFAULT
+    # constraint in the expression that is generated here.
+    if isinstance(server_default, bool):
+        server_default = None
+    return "%s %s %s%s%s" % (
+        alter_table(compiler, table_name, schema),
+        alter_column(compiler, column_name),
+        format_type(compiler, type_),
+        "" if nullable else " NOT NULL",
+        (
+            ""
+            if server_default is None
+            else f" DEFAULT {format_server_default(compiler, server_default)}"
+        ),
     )
